#ifndef seafire__routing__host_parameter_hxx_
#define seafire__routing__host_parameter_hxx_

#include <seafire/server/parameters.hxx>
#include <seafire/server/request.hxx>

#include <seafire/routing/parameters.hxx>

#include <optional>

namespace seafire::routing
{

  template<
    server::parameter_name_t Name,
    typename ParameterType = server::string_parameter_t
  >
  class host_parameter_t
  {
  public:
    using parameter_type = ParameterType;
    using value_type = typename parameter_type::value_type;

    static
    std::string const&
    name()
    {
      static std::string const name{Name};
      return name;
    }

    host_parameter_t(std::optional<value_type> value)
      : _value{std::move(value)}
    {}

    std::optional<value_type> const&
    value() const
    {
      return _value;
    }

    operator std::optional<value_type> const&() const
    {
      return value();
    }

    std::optional<value_type> const*
    operator->() const
    {
      return &value();
    }

    static
    host_parameter_t<Name, ParameterType>
    fetch(server::request_t& req)
    {
      auto v = req.extensions().use<host_parameters_t>().get(name());
      return parameter_type::try_parse(v);
    }

  private:
    std::optional<value_type> _value;

  };

} // namespace seafire::routing

#endif
